@@ -388,7 +388,7 @@ class Automation:
             return {'return':16, 'error':'repository {} not found'.format(artifact_repo)}
 
         if len(lst)>1:
-            return {'return':1, 'error':'more than 1 repository found'}
+            return {'return':1, 'error':'more than 1 repository found (ambiguity)'}
 
         repo = lst[0]
 
@@ -431,7 +431,7 @@ class Automation:
         existing_tags = meta.get('tags',[])
         if len(tags)>0: 
             existing_tags += tags
-        meta['tags'] = existing_tags
+        meta['tags'] = utils.filter_tags(existing_tags)
 
         # Record meta
         r = utils.save_json(meta_path_json, meta=meta)
@@ -613,7 +613,7 @@ class Automation:
             (meta) (dict): new meta description to be merged or to replace the original meta description of a CM artifact
 
             (tags) (string or list): tags to be added to meta
-            
+
             (replace) (bool): if True, replace original meta description (False by default)
             (force) (bool): if True, force updates if more than 1 artifact found (False by default)
             (replace_lists) (bool): if True, replace lists in meta description rather than appending them (False by default)
@@ -748,29 +748,32 @@ class Automation:
         if len(lst)==0:
             return {'return':16, 'error':'artifact not found: {}'}
 
-        # Check target repo
+        # Check target artifact
         target_artifact = parsed_artifacts[0]
 
         target_artifact_obj = target_artifact[0]
-        target_artifact_repo = target_artifact[1] if len(target_artifact)>0 else None
-
-        r = self.cmind.access({'action':'search',
-                               'automation':'repo',
-                               'artifact': utils.assemble_cm_object2(target_artifact_repo)})
-        if r['return']>0: return r
-
-        target_repo_list = r['list']
-
-        if len(target_repo_list) == 0:
-            return {'return':1, 'error':'target repo "{}" not found'.format(target_artifact_repo)}
-        elif len(target_repo_list) >1:
-            return {'return':1, 'error':'more than 1 target repo found "{}"'.format(target_artifact_repo)}
-
-        target_repo_path = os.path.abspath(target_repo_list[0].path)
-
         target_artifact_obj_alias = target_artifact_obj[0]
         target_artifact_obj_uid = target_artifact_obj[1].lower()
 
+        # Check target repo
+        target_artifact_repo = target_artifact[1] if len(target_artifact)>1 else None
+
+        target_repo_path = None
+
+        if target_artifact_repo is not None:
+            r = self.cmind.access({'action':'search',
+                                   'automation':'repo',
+                                   'artifact': utils.assemble_cm_object2(target_artifact_repo)})
+            if r['return']>0: return r
+
+            target_repo_list = r['list']
+
+            if len(target_repo_list) == 0:
+                return {'return':1, 'error':'target repo "{}" not found'.format(target_artifact_repo)}
+            elif len(target_repo_list) >1:
+                return {'return':1, 'error':'more than 1 target repo found "{}"'.format(target_artifact_repo)}
+
+            target_repo_path = os.path.abspath(target_repo_list[0].path_with_prefix)
 
         # Updating artifacts
         for artifact in lst:
@@ -793,7 +796,9 @@ class Automation:
             if target_artifact_obj_alias != '':
                 new_name = target_artifact_obj_alias
 
-            new_artifact_path = os.path.join(target_repo_path, artifact_automation, new_name)
+            # Use either new repo or the repo of the original artifact
+            new_artifact_path = os.path.join(target_repo_path, artifact_automation, new_name) if target_repo_path is not None \
+                else os.path.join(artifact_automation_dir, new_name)
 
             # Check if need to update meta
             if target_artifact_obj_alias != '' and artifact_alias.lower() != target_artifact_obj_alias.lower():
@@ -812,6 +817,9 @@ class Automation:
                     print ('* Moving "{}" to'.format(artifact_path))
                     print ('         "{}"'.format(new_artifact_path))
 
+                if os.path.isdir(new_artifact_path):
+                    return {'return':1, 'error':'target artifact (directory) already exists'}
+
                 shutil.move(artifact_path, new_artifact_path)
 
             # Update meta
@@ -823,3 +831,214 @@ class Automation:
                 if r['return'] >0: return r
 
         return {'return':0, 'list':lst}
+
+    ############################################################
+    def copy(self, i):
+        """
+        Copy CM artifact(s) either to a new artifact or to a new repository
+
+        Args:
+            (CM input dict):
+
+            (out) (str): if 'con', output to console
+
+            parsed_automation (list): prepared in CM CLI or CM access function
+                                      [ (automation alias, automation UID) ] or
+                                      [ (automation alias, automation UID), (automation repo alias, automation repo UID) ]
+
+            parsed_artifact (list): prepared in CM CLI or CM access function
+                                      [ (artifact alias, artifact UID) ] or
+                                      [ (artifact alias, artifact UID), (artifact repo alias, artifact repo UID) ]
+
+            parsed_artifacts (list): prepared in CM CLI or CM access function - 1st entry has a new artifact name and repository:
+                                      [ (artifact alias, artifact UID) ] or
+                                      [ (artifact alias, artifact UID), (artifact repo alias, artifact repo UID) ]
+
+        Returns: 
+            (CM return dict):
+
+            * return (int): return code == 0 if no error and >0 if error
+            * (error) (str): error string if return>0
+
+            * list (list): list of renamed/moved CM artifacts
+
+        """
+
+        import shutil
+
+        console = i.get('out') == 'con'
+
+        parsed_artifacts = i.get('parsed_artifacts',[])
+
+        if len(parsed_artifacts)==0:
+           return {'return':1, 'error':'artifact is not specified'}
+        elif len(parsed_artifacts)>1:
+           return {'return':1, 'error':'more than 1 artifact specified'}
+
+        # Find CM artifact(s)
+        i['out'] = None
+        r = self.search(i)
+        if r['return']>0: return r
+
+        lst = r['list']
+        if len(lst)==0:
+            return {'return':16, 'error':'artifact(s) not found: {}'}
+
+        # Check target artifact
+        target_artifact = parsed_artifacts[0]
+
+        target_artifact_obj = target_artifact[0]
+        target_artifact_obj_alias = target_artifact_obj[0]
+        target_artifact_obj_uid = target_artifact_obj[1].lower()
+
+        # Check target repo
+        target_artifact_repo = target_artifact[1] if len(target_artifact)>1 else None
+
+        target_repo_path = None
+
+        if target_artifact_repo is not None:
+            print (target_artifact_repo)
+            r = self.cmind.access({'action':'search',
+                                   'automation':'repo',
+                                   'artifact': utils.assemble_cm_object2(target_artifact_repo)})
+            if r['return']>0: return r
+
+            target_repo_list = r['list']
+
+            if len(target_repo_list) == 0:
+                return {'return':1, 'error':'target repo "{}" not found'.format(target_artifact_repo)}
+            elif len(target_repo_list) >1:
+                return {'return':1, 'error':'more than 1 target repo found "{}"'.format(target_artifact_repo)}
+
+            target_repo_path = os.path.abspath(target_repo_list[0].path_with_prefix)
+
+        # Updating artifacts
+        for artifact in lst:
+
+            artifact_path = os.path.abspath(artifact.path)
+
+            artifact_meta = artifact.original_meta
+
+            artifact_alias = artifact_meta.get('alias','')
+            r=utils.gen_uid()
+            if r['return']>0: return r
+
+            target_artifact_obj_uid=r['uid']
+
+            artifact_dir_name = os.path.basename(artifact_path)
+            artifact_automation_dir = os.path.dirname(artifact_path)
+            artifact_automation = os.path.basename(artifact_automation_dir)
+
+            # Prepare new path
+            new_name = artifact_dir_name
+            if target_artifact_obj_alias != '':
+                new_name = target_artifact_obj_alias
+
+            # Use either new repo or the repo of the original artifact
+            new_artifact_path = os.path.join(target_repo_path, artifact_automation, new_name) if target_repo_path is not None \
+                else os.path.join(artifact_automation_dir, new_name)
+
+            if target_artifact_obj_alias != '' and artifact_alias.lower() != target_artifact_obj_alias.lower():
+                artifact_meta['alias']=target_artifact_obj_alias
+
+            artifact_meta['uid']=target_artifact_obj_uid
+
+            artifact.path = new_artifact_path
+
+            # Copy
+            if artifact_path != new_artifact_path:
+                if console:
+                    print ('* Copying "{}" to'.format(artifact_path))
+                    print ('         "{}"'.format(new_artifact_path))
+
+                if os.path.isdir(new_artifact_path):
+                    return {'return':1, 'error':'target artifact (directory) already exists'}
+
+                shutil.copytree(artifact_path, new_artifact_path)
+
+            # Update meta
+            if console:
+                print ('- Updating meta in "{}"'.format(new_artifact_path))
+
+            r = artifact.update({})
+            if r['return'] >0: return r
+
+        return {'return':0, 'list':lst}
+
+    ############################################################
+    def info(self, i):
+        """
+        Get info about artifacts
+
+        Args:
+            (CM input dict):
+
+            (out) (str): if 'con', output to console
+
+            parsed_automation (list): prepared in CM CLI or CM access function
+                                      [ (automation alias, automation UID) ] or
+                                      [ (automation alias, automation UID), (automation repo alias, automation repo UID) ]
+
+            parsed_artifact (list): prepared in CM CLI or CM access function
+                                      [ (artifact alias, artifact UID) ] or
+                                      [ (artifact alias, artifact UID), (artifact repo alias, artifact repo UID) ]
+
+            parsed_artifacts (list): prepared in CM CLI or CM access function - 1st entry has a new artifact name and repository:
+                                      [ (artifact alias, artifact UID) ] or
+                                      [ (artifact alias, artifact UID), (artifact repo alias, artifact repo UID) ]
+
+        Returns: 
+            (CM return dict):
+
+            * return (int): return code == 0 if no error and >0 if error
+            * (error) (str): error string if return>0
+
+            * list (list): list of renamed/moved CM artifacts
+
+        """
+
+        # Check parsed automation
+        if 'parsed_automation' not in i:
+           return {'return':1, 'error':'automation is not specified'}
+
+        console = i.get('out') == 'con'
+
+        # Find CM artifact(s)
+        i['out'] = None
+        r = self.search(i)
+
+        if r['return']>0: return r
+
+        lst = r['list']
+        if len(lst)==0:
+            return {'return':16, 'error':'artifact not found: {}'.format(i)}
+
+        for artifact in lst:
+            if console and len(lst)>1:
+                print ('****************************************************')
+
+            path = artifact.path
+            meta = artifact.meta
+            original_meta = artifact.original_meta
+
+            alias = meta.get('alias','')
+            uid = meta.get('uid','')
+            automation_alias = meta.get('automation_alias','')
+            automation_uid = meta.get('automation_uid','')
+
+            cid = utils.assemble_cm_object(automation_alias,automation_uid) + \
+                  '::' + \
+                  utils.assemble_cm_object(alias,uid)
+            
+#            cid_user_friendly = automation_alias if automation_alias != '' else automation_uid
+#            cid_user_friendly += '::' + (alias if alias != '' else uid)
+#            cid_misc = automation_alias if automation_alias != '' else automation_uid
+#            cid_misc += '::' + uid
+#            cid = automation_uid + '::' + uid
+
+            # Output if console
+            if console:
+                print (cid)
+                print ('  '+path)
+
+        return {'return':0, 'list': lst}
