@@ -232,7 +232,11 @@ def load_yaml(file_name, check_if_exists = False, encoding = 'utf8'):
     import yaml
 
     with open(file_name, 'rt', encoding = encoding) as yf:
-        meta = yaml.load(yf, Loader=yaml.FullLoader)
+        if int(yaml.__version__[0])>=5:
+            meta = yaml.load(yf, Loader=yaml.FullLoader)
+        else:
+            # To support old versions
+            meta = yaml.safe_load(yf)
 
     return {'return':0,
             'meta': meta}
@@ -493,8 +497,6 @@ def list_all_files(i):
         pe = i['path_ext']
 
     po = i.get('path', '')
-    if sys.version_info[0] < 3:
-        po = unicode(po)
 
     pattern = i.get('pattern', '')
     if pattern != '':
@@ -514,7 +516,7 @@ def list_all_files(i):
     else:
         for fn in dirList:
             p = os.path.join(po, fn)
-            if iall == 'yes' or fn not in cfg['special_directories']:
+            if iall == 'yes':
                 if len(inames) == 0 or fn not in inames:
                     if os.path.isdir(p):
                         if not isd or os.path.realpath(p) == p:
@@ -1383,3 +1385,19 @@ def convert_env_to_dict(s):
                 d[k]=v
 
     return {'return':0, 'dict':d}
+
+###########################################################################
+def filter_tags(tags):
+    """
+    Filter tags (remove starting with -)
+
+    Args:    
+       tags (list): list of tags
+
+    Returns: 
+       (list) list of filtered tags
+    """
+
+    filtered_tags = [t for t in tags if not t.startswith('-')]
+
+    return filtered_tags
